-- Divisor  by Vger-Azjol-Nerub
-- www.vgermods.com
-- 
-- Version 1.0: first official release of Divisor
--
------------------------------------------------------------

-- Divisor requires this version of VgerCore:
local DivisorVgerCoreVersionRequired = 0.41

-- Frames
local DivisorCoreFrame = nil


------------------------------------------------------------
-- Divisor events
------------------------------------------------------------

-- Called when an event that Divisor cares about is fired.
function DivisorOnEvent(Event, Unused1, Unused2, Unused3)
	if Event == "VARIABLES_LOADED" then 
		DivisorInitialize()
	end 
end

-- Initializes Pawn after all saved variables have been loaded.
function DivisorInitialize()
	VgerCore.Message(VgerCore.Color.Blue .. DivisorLocal.LoadedMessage)
	
	-- Check the current version of VgerCore.
	if (not VgerCore) or (not VgerCore.Version) or (VgerCore.Version < DivisorVgerCoreVersionRequired) then
		VgerCore.Message(VgerCore.Color.Salmon .. DivisorLocal.NeedNewerVgerCoreMessage)
		message(DivisorLocal.NeedNewerVgerCoreMessage)
	end

	SLASH_DIVISOR1 = "/divisor"
	SlashCmdList["DIVISOR"] = DivisorCommand

	-- Set any unset options to their default values.  If the user is a new Divisor user, all options
	-- will be set to default values.  If upgrading, only missing options will be set to default values.
	DivisorSetEmptyOptions()
	
	-- Hook into events.
	VgerCore.HookInsecureFunction("AuctionFrameBrowse_Update", DivisorAuctionFrameUpdate)
	for i = 1, NUM_BROWSE_TO_DISPLAY do
		VgerCore.HookInsecureScript(getglobal("BrowseButton" .. i), "OnEnter", function() DivisorAuctionItem_OnEnter(i) end)
		VgerCore.HookInsecureScript(getglobal("BrowseButton" .. i), "OnLeave", function() DivisorAuctionItem_OnLeave(i) end)
	end

end

-- Resets all Divisor options.  Used to set the saved variable to a default state.
function DivisorResetOptions()
	DivisorOptions = nil
	DivisorSetEmptyOptions();
end

-- Sets values for any options that don't have a value set yet.  Useful when upgrading.
function DivisorSetEmptyOptions()
	if not DivisorOptions then DivisorOptions = {} end
	
	if not DivisorOptions.Stacks then DivisorOptions.Stacks = DivisorGetDefaultStacks() end
end

-- Called after AuctionFrameBrowse_Update returns.
function DivisorAuctionFrameUpdate()
	GameTooltip:Hide()
end

-- Called after the default OnEnter handler for AuctionButton{Index}.
function DivisorAuctionItem_OnEnter(Index)
	local Button = getglobal("BrowseButton" .. Index)
	
	local Offset = FauxScrollFrame_GetOffset(BrowseScrollFrame)
	local Name, _, Quantity, Quality, _, _, MinimumBid, MinimumIncrement, BuyoutPrice, BidAmount, IsHighBidder = GetAuctionItemInfo("list", Offset + Index)
	local DesiredStackSize = DivisorOptions.Stacks[Name]
	
	-- Prepare the tooltip.
	local AddedLine
	
	-- Add the prices for a single of this item.
	AddedLine = DivisorAnnotateAuctionTooltip(GameTooltip, Button, AddedLine, 1,
		Name, Quantity, Quality, MinimumBid, MinimumIncrement, BuyoutPrice, BidAmount, IsHighBidder)
	
	-- For certain items, we care about another stack size besides 1.
	AddedLine = DivisorAnnotateAuctionTooltip(GameTooltip, Button, AddedLine, DesiredStackSize,
		Name, Quantity, Quality, MinimumBid, MinimumIncrement, BuyoutPrice, BidAmount, IsHighBidder)
	
	-- Show the finished tooltip.  Don't do this until all modifications are complete.  Also, don't do this if we didn't
	-- have any annotations to make.
	if AddedLine then GameTooltip:Show() end
end

-- Called after the default OnLeave handler for AuctionButton{Index}.
function DivisorAuctionItem_OnLeave(Index)
	-- NOTE: This function is not called when the display is scrolled, but DivisorAuctionFrameUpdate is.
	GameTooltip:Hide()
end

------------------------------------------------------------
-- Divisor core methods
------------------------------------------------------------

-- Processes a Divisor slash command.
function DivisorCommand(Command)
	if Command == DivisorLocal.ResetOptionsCommand then
		DivisorResetOptions()
		VgerCore.Message("Divisor options have been reset.")
	elseif Command == DivisorLocal.ListStacksCommand then
		DivisorListStacks()
	elseif string.sub(Command, 1, string.len(DivisorLocal.SetStackCommand)) == DivisorLocal.SetStackCommand then
		local StackLine = string.trim(string.sub(Command, string.len(DivisorLocal.SetStackCommand) + 1))
		local Pos, _, Quantity, ItemLink = string.find(StackLine, "^%s-(%d+)%s+(.-)$")
		Quantity = tonumber(Quantity)
		ItemLink = DivisorGetItemNameFromLink(ItemLink)
		if Quantity and Quantity > 0 and ItemLink then
			DivisorAddStack(ItemLink, Quantity)
		else
			VgerCore.Message(VgerCore.Color.Salmon .. DivisorLocal.SetStackUsage)
		end
	elseif string.sub(Command, 1, string.len(DivisorLocal.DeleteStackCommand)) == DivisorLocal.DeleteStackCommand then
		local ItemLink = DivisorGetItemNameFromLink(string.trim(string.sub(Command, string.len(DivisorLocal.DeleteStackCommand) + 1)))
		if ItemLink then
			DivisorDeleteStack(ItemLink)
		else
			VgerCore.Message(VgerCore.Color.Salmon .. DivisorLocal.DeleteStackUsage)
		end
	else
		DivisorUsage()
	end
end

-- Displays Divisor usage information.
function DivisorUsage()
	VgerCore.Message(" ")
	VgerCore.MultilineMessage(DivisorLocal.Usage)
	VgerCore.Message(" ")
end

-- Returns the item name from an item link, or if the string isn't an item link, returns the original string.
function DivisorGetItemNameFromLink(ItemLink)
	if not ItemLink then return nil end
	local Pos, _, ItemName = string.find(ItemLink, "^|c%x+|H.+|h%[(.+)%]|h|r$")
	if ItemName then return ItemName else return ItemLink end
end

-- Lists all of the stack sizes that are set, including the default ones.
function DivisorListStacks()
	for ItemName, Quantity in pairs(DivisorOptions.Stacks) do
		VgerCore.Message(string.format(DivisorLocal.ListStacksStackMessage, Quantity, ItemName))
	end
end

-- Adds an alternate stack size (other than 1) to a particular item.
function DivisorAddStack(ItemName, Quantity)
	DivisorOptions.Stacks[ItemName] = Quantity
	VgerCore.Message(string.format(DivisorLocal.SetStackMessage, ItemName, Quantity))
end

-- Removes the alternate stack size for an item, so Divisor will only show individual prices.
function DivisorDeleteStack(ItemName)
	DivisorOptions.Stacks[ItemName] = nil
	VgerCore.Message(string.format(DivisorLocal.DeleteStackMessage, ItemName))
end

-- Annotates an auction house UI tooltip.
-- Tooltip: the tooltip instance (such as GameTooltip) to modify.
-- TooltipOwner: the object that should own the tooltip.
-- AddedLine: true if we've already added one annotation to this tooltip already, or false if we still need to set it up.
-- TooltipQuantity: the number of the item you want to display on the tooltip.  Generally 1.
-- (Other parameters): the return values from GetAuctionItemInfo or an equivalent function.
-- Return value: true if an annotation was added OR if AddedLine was true.  The caller should set its own AddedLine to this value.
function DivisorAnnotateAuctionTooltip(Tooltip, TooltipOwner, AddedLine, TooltipQuantity, Name, Quantity, Quality, MinimumBid, MinimumIncrement, BuyoutPrice, BidAmount, IsHighBidder)
	-- If the tooltip quantity is invalid or matches the actual quantity, exit.
	if (not TooltipQuantity) or (TooltipQuantity == Quantity) then return AddedLine end

	-- Prepare the tooltip.
	if AddedLine then
		Tooltip:AddLine(" ")
	else
		Tooltip:SetOwner(TooltipOwner, "ANCHOR_RIGHT", 0, -AUCTIONS_BUTTON_HEIGHT)
		Tooltip:ClearLines()
	end
	
	-- Calculate the actual minimum bid.
	-- If you're the high bidder, then there's no reason to add the increment, because you can't rebid.
	local ActualMinimumBid = math.max(BidAmount, MinimumBid)
	if not IsHighBidder then ActualMinimumBid = ActualMinimumBid + MinimumIncrement end
	
	Tooltip:AddLine(tostring(TooltipQuantity) .. " " .. ITEM_QUALITY_COLORS[Quality].hex .. Name)

	if ActualMinimumBid > 0 and (BuyoutPrice == 0 or ActualMinimumBid < BuyoutPrice) then
		local Caption
		if IsHighBidder then Caption = DivisorLocal.YourBid else Caption = DivisorLocal.MinimumBid end
		Tooltip:AddDoubleLine(Caption, FormatMoney(ceil(TooltipQuantity * ActualMinimumBid / Quantity), true))
	end
	if BuyoutPrice > 0 then
		Tooltip:AddDoubleLine(DivisorLocal.Buyout, FormatMoney(ceil(TooltipQuantity * BuyoutPrice / Quantity), true))
	end
	return true
end

-- Returns a colored string that represents the amount of money specified, such as "2g 50s".
-- If LongForm is true, silver is always present when gold is present, and copper is always present when silver is present.
FormatMoney_Gold = VgerCore.Color.White .. "%d" .. VgerCore.MoneyColor.Gold .. DivisorLocal.GoldLetter .. " "
FormatMoney_Silver = VgerCore.Color.White .. "%d" .. VgerCore.MoneyColor.Silver .. DivisorLocal.SilverLetter .. " "
FormatMoney_Copper = VgerCore.Color.White .. "%d" .. VgerCore.MoneyColor.Copper .. DivisorLocal.CopperLetter .. " "
function FormatMoney(TotalCopper, LongForm)
	local Gold = floor(TotalCopper / (COPPER_PER_SILVER * SILVER_PER_GOLD))
	local Silver = floor((TotalCopper - (Gold * COPPER_PER_SILVER * SILVER_PER_GOLD)) / COPPER_PER_SILVER)
	local Copper = mod(TotalCopper, COPPER_PER_SILVER)
	local FinalString = VgerCore.Color.White
	
	if Gold > 0 then FinalString = FinalString .. string.format(FormatMoney_Gold, Gold) end
	if Silver > 0 or (LongForm and Gold > 0) then FinalString = FinalString .. string.format(FormatMoney_Silver, Silver) end
	if Copper > 0 or (not (Gold > 0 or Silver > 0)) or (LongForm and (Gold > 0 or Silver > 0)) then FinalString = FinalString .. string.format(FormatMoney_Copper, Copper) end
	
	return string.sub(FinalString, 0, -1) .. VgerCore.Color.Reset
end

------------------------------------------------------------

-- Core frame setup
if (not DivisorCoreFrame) then
	DivisorCoreFrame = CreateFrame("Frame", "DivisorCoreFrame")
end

DivisorCoreFrame:SetScript("OnEvent", function() DivisorOnEvent(event, arg1, arg2, arg3) end)

DivisorCoreFrame:RegisterEvent("VARIABLES_LOADED")
